"use strict";
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const _ = require("lodash");
const CryptoJS = require("crypto-js");
/**
 * Decrypt Fields
 * --------------
 * Decrypt one or more fields using the shared secret
 *
 * @type {{name: string, title: string, when: [string,string], fn: module.exports.fn, Decrypter: module.exports.Decrypter}}
 */
class decrypt {
    constructor() {
        this.name = "api.decrypt";
        this.title = "Return payload with specified fields decrypted";
    }
    fn(operation, _options) {
        let context = operation.context;
        assert(context, "missing context");
        let options = _.extend({ algo: "AES", secret: "", cipherField: "_encrypted_" }, _options, operation.feature);
        let cipher = CryptoJS[options.algo];
        assert(cipher, "missing cipher: " + options.algo);
        return function (req, res, next) {
            assert(req && res && next, "invalid middleware");
            let model = req.body;
            let cooked = model[options.cipherField];
            if (cooked) {
                let raw = cipher.decrypt(cooked, options.secret);
                var decryped = JSON.parse(raw.toString(CryptoJS.enc.Utf8));
                _.extend(model, decryped);
                delete model[options.cipherField];
                // req.body = req.json = model;
                // res.model = res.json = model;
                res.send(model);
                // next();
            }
            else {
                next();
            }
        };
    }
    ;
}
exports.default = decrypt;
//# sourceMappingURL=decrypt.js.map