"use strict";

/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import * as core  from "api-service-core";

import {  amqp, cors, harden, KeycloakPlugin, remote_config, prometheus, ua, render, webhooks, UMA } from "./plugins";
export {  amqp, cors, harden, KeycloakPlugin, remote_config, prometheus, ua, render, webhooks, UMA } from "./plugins";
import { apidocs, api_config, decrypt, encrypt, local_file, redact, redirect, static_assets } from "./middleware";
export { apidocs, api_config, decrypt, encrypt, local_file, redact, redirect, static_assets } from "./middleware";

export { Chassis, IChassisPlugin, IOperation, IChassisFeature, IChassisDataStore, IChassisConfig, IChassisContext, IChassisMiddleware, IChassisLogger } from "api-service-core";

/*
        Register all our default plugins
        These can are installed before the server starts listening
 */
let default_plugins = function() {
    this.registerPlugin( new amqp );
    this.registerPlugin( new cors );
    this.registerPlugin( new harden );
    this.registerPlugin( new KeycloakPlugin );
    this.registerPlugin( new UMA );
    this.registerPlugin( new prometheus );
    this.registerPlugin( new ua );
    this.registerPlugin( new render );
    this.registerPlugin( new remote_config );
    this.registerPlugin( new webhooks );

    this.registerPlugin( new core.PipelinePlugin );
    this.registerPlugin( new core.after );
    this.registerPlugin( new core.before );
    this.registerPlugin( new core.graphql );
    this.registerPlugin( new core.logging );
    this.registerPlugin( new core.jwt );
    this.registerPlugin( new core.payload );
    this.registerPlugin( new core.OpenAPIPlugin );
}

/*
        Register all our middleware plugins
        These can be attached as chassis:operations to Open API specs
 */
let default_middleware = function() {
    this.registerFn( new core.bower_components );
    this.registerFn( new core.debug );
    this.registerFn( new core.DataDict );
    this.registerFn( new core.echo );
    this.registerFn( new core.gregarious );
    this.registerFn( new core.heartbeat );
    this.registerFn( new core.headers );
    this.registerFn( new core.mock );
    this.registerFn( new core.mount );
    this.registerFn( new core.not_found );
    this.registerFn( new core.proxy );
    this.registerFn( new core.request_uuid );
    this.registerFn( new core.sortby );

    this.registerFn( new apidocs);
    this.registerFn( new decrypt );
    this.registerFn( new encrypt );
    this.registerFn( new local_file );
    this.registerFn( new api_config );
    this.registerFn( new redact );
    this.registerFn( new redirect );
    this.registerFn( new static_assets );
}

export function default_features() {
    default_plugins.apply(this);
    default_middleware.apply(this);
}
    
