/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import * as assert from "assert";
import * as _ from "lodash";
import * as CryptoJS from "crypto-js";
import { IOperation, IChassisContext, IChassisMiddleware, Vars} from "api-service-core";
import { Response, NextFunction } from "express";

/**
 * Encrypt Fields
 * --------------
 * Encrypt one or more model using the shared secret
 *
 * @type {{name: string, title: string, when: [string,string], fn: module.exports.fn, Encrypter: module.exports.Encrypter}}
 */
export default class encrypt implements IChassisMiddleware {

    name = "api.encrypt";
    title = "Return payload with specified model encrypted";

    fn(operation: IOperation, _options: any): Function {
        let context: IChassisContext = operation.context;

        assert(context, "missing context");

        let options = _.extend({ fields: [], redact: true, algo: "AES", secret: "", cipherField: "_encrypted_" }, _options, operation.feature);
        let cipher = CryptoJS[options.algo];
        assert(cipher, "missing cipher: "+options.algo);

        return ( (req: any, res: Response, next: NextFunction) => {
            assert( req && res && next, "invalid middleware");

            let model = req.json || req.body || {};

            let fields = _.pick(model, options.fields);
            let raw = JSON.stringify(fields);
            let cooked = cipher.encrypt( raw.toString(), options.secret ).toString();
            model[options.cipherField] = cooked;
            if (options.redact) {
                Vars.redact(model, options.fields);
            }

            req.body = req.json = model;
            next();
        });
    };

};
