/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import * as assert from "assert";
import { IOperation, IChassisContext, IChassisFeature } from "api-service-core";
import * as express from "express";
// import * as serveStatic from "serve-static";
import * as send from "send";

/**
 * Static assets
 * -------------
 * Attach an API resource to static assets from file system
 *
 *  static_assets:
 *      path:  {current_folder}+"./public"/
 *
 * @type {{name: string, title: string, description: string, fn: module.exports.fn}}
 */
export default class static_assets implements IChassisFeature {
    name = "api.static_assets";
    title = "static assets";

    fn(operation: IOperation, options: any): Function {
        let context: IChassisContext = operation.context;
        assert(context, "missing context");

        operation.actionId = "get";
        operation.spec.description = this.title;
        if (!operation.spec.responses) {
            operation.spec.responses[200] = { description: "mixed assets", content: { "*/*": {} }};
        }

        // operation = this._wildcard(operation);
        let folder = options.folder || process.cwd()+"/public";
        let prefix = operation.resource;
        if (prefix.indexOf("/*")) prefix = prefix.substring(0,prefix.indexOf("/*"));
        assert(operation.resource.startsWith(prefix), "invalid mount for static assets");

        operation.context.log({ code: "api:static_assets", resource: operation.resource, folder: folder });

        return (req: express.Request, res: express.Response, next?: express.NextFunction)=> {
            let path = req.path.substring(prefix.length);
//            operation.context.log({ code: "api:static_asset", path: path, folder: folder, resource: operation.resource });

            send(req, path, { root: folder }).pipe(res).on("directory", () => {
                next();
            }).on('error',(err)=>{
                if ((err.statusCode < 500)) next()
                else next(err);
            });
        }
    }

};